      SUBROUTINE STKPAR
C
C Parse planarity restraints
C Author: Martin Laurberg. Based on PLNPAR: Peter Schultze 
C
      IMPLICIT NONE
C I/O
      INCLUDE 'cns.inc'
      INCLUDE 'comand.inc'
      INCLUDE 'heap.inc'
      INCLUDE 'funct.inc'
      INCLUDE 'stack.inc'
      INCLUDE 'mtf.inc'
C local
      INTEGER STKFLG, POINTR, STKFL2, POINT2
C
C begin
C allocate space on heap for temporary pointer arrays
C      STKFLG=ALLHP(INTEG4(NATOM))
C      POINTR=ALLHP(INTEG4(NATOM))
C      CALL RESSTK(HEAP(STKFLG),HEAP(POINTR))
C
      STKFLG=ALLHP(INTEG4(NATOM))
      POINTR=ALLHP(INTEG4(NATOM))
      STKFL2=ALLHP(INTEG4(NATOM))
      POINT2=ALLHP(INTEG4(NATOM))
      CALL RESSTK(HEAP(STKFLG),HEAP(POINTR),
     &            HEAP(STKFL2),HEAP(POINT2))
C
C
C free space on heap
      CALL FREHP(POINTR,INTEG4(NATOM))
      CALL FREHP(STKFLG,INTEG4(NATOM))
C
      CALL FREHP(POINT2,INTEG4(NATOM))
      CALL FREHP(STKFL2,INTEG4(NATOM))

      RETURN
      END
C
C----------------------------------------------------------------------
      SUBROUTINE STKINR
C
C Initialization of planarity restraints
C
C Author: Martin Laurberg. Based on PLNINR: Peter Schultze 
C
      IMPLICIT NONE
C I/O
      INCLUDE 'stack.inc'
C local
      INTEGER K
C begin
C
      LSTKRE=.FALSE.
C Restraints
      NGRUPQ=0
C initialize heap pointers
      DO K=1,MAXSGR
      HPSSTP(K)=0
      LPSSTP(K)=0
C
      HPSST2(K)=0
      LPSST2(K)=0
C
      END DO
      RETURN
      END
C
C---------------------------------------------------------------------
      SUBROUTINE RESSTK(STKFLG,POINTR,STKFL2,POINT2)
C
C Parsing routine to set up pointer array POINTR to reference
C groups of atoms to be restrained to planarity.
C
C Author: Martin Laurberg. Based on RESPLN: Peter Schultze 
C
      IMPLICIT NONE
C I/O
      INCLUDE 'cns.inc'
      INCLUDE 'comand.inc'
      INCLUDE 'coord.inc'
      INCLUDE 'heap.inc'
      INCLUDE 'funct.inc'
      INCLUDE 'stack.inc'
      INCLUDE 'mtf.inc'
      INTEGER STKFLG(*), POINTR(*),STKFL2(*), POINT2(*)
C local
      INTEGER I,NSELEC
C
C begin parsing
      CALL PUSEND('STAK>')
      DO WHILE (.NOT.DONE)
      CALL NEXTWD('STAK>')
      CALL MISCOM('STAK>',USED)
      IF (.NOT.USED) THEN
C
C====================================================================
      IF (WD(1:4).EQ.'HELP') THEN
C
      CALL CNSHELP('cns-restraints-plane')
C
      ELSE IF (WD(1:4).EQ.'INIT') THEN
C
C free-up HEAP space if used before
      CALL STKFIN
      CALL STKINR
C=====================================================================
      ELSE IF (WD(1:4).EQ.'GROU') THEN
C
C turn on STK restraints flag
      LSTKRE=.TRUE.
C
C make new group
      IF (NGRUPQ.GT.MAXSGR) THEN
      CALL WRNDIE(-5,'STAK',
     & 'MAXSGR (max. no. of STAK groups) exceeded')
      ELSE
      NGRUPQ=NGRUPQ+1
      END IF
C
C fill default values 
      NATSTK(NGRUPQ)=0
      NATST2(NGRUPQ)=0
      WTSTK(NGRUPQ)=300.0D0
C
C ccccccccccccccccccccccccccccccccccccccccccccccccccccc
C fill default values for center-to-center parameters
      DCEN(NGRUPQ)=4.0D0
      LCEN(NGRUPQ)=1.5D0
      UCEN(NGRUPQ)=1.5D0
      WCEN(NGRUPQ)=300.0D0
C fill default values for center-to-plane parameters
      DPLA(NGRUPQ)=3.0D0
      LPLA(NGRUPQ)=0.4D0
      UPLA(NGRUPQ)=0.4D0
      WPLA(NGRUPQ)=300.0D0
C cccccccccccccccccccccccccccccccccccccccccccccccccccccc
C
      CALL PUSEND('STAK-GROUp>')
      DO WHILE (.NOT.DONE)
      CALL NEXTWD('STAK-GROUp>')
      CALL MISCOM('STAK-GROUp>',USED)
      IF (.NOT.USED) THEN
      IF (WD(1:4).EQ.'HELP') THEN
C
      CALL CNSHELP('cns-restraints-plane-group')
C
      ELSE IF (WD(1:4).EQ.'SELA') THEN
      CALL SELCTA(STKFLG,NSELEC,X,Y,Z,.TRUE.)
      CALL MAKIND(STKFLG,NATOM,NSELEC)
      NATSTK(NGRUPQ)=NSELEC
C
C fill temporary pointer array
      DO I=1,NATSTK(NGRUPQ)
      POINTR(I)=STKFLG(I)
      END DO
C
      ELSE IF (WD(1:4).EQ.'SELB') THEN
      CALL SELCTA(STKFL2,NSELEC,X,Y,Z,.TRUE.)
      CALL MAKIND(STKFL2,NATOM,NSELEC)
      NATST2(NGRUPQ)=NSELEC
C
C fill temporary pointer array
      DO I=1,NATST2(NGRUPQ)
      POINT2(I)=STKFL2(I)
      END DO
C
      ELSE IF (WD(1:4).EQ.'WEIG') THEN
      CALL NEXTF('WEIGht=',WTSTK(NGRUPQ))
C
C cccccccccccccccccccccccccccccccccccccccccccccccccccccc
C Parse center-to-center distance restraint parameters
      ELSE IF (WD(1:4).EQ.'DCEN') THEN
      CALL NEXTF('Distance center-to-center=',DCEN(NGRUPQ))
      CALL NEXTF('lower-average-distance=',LCEN(NGRUPQ))
      CALL NEXTF('upper-average-distance=',UCEN(NGRUPQ))
      ELSE IF (WD(1:4).EQ.'WCEN') THEN
      CALL NEXTF('Weight distance cen2cen=',WCEN(NGRUPQ))
C Parse center-to-plane distance restraint parameters
      ELSE IF (WD(1:4).EQ.'DPLA') THEN
      CALL NEXTF('Distance center-to-center=',DPLA(NGRUPQ))
      CALL NEXTF('lower-average-distance=',LPLA(NGRUPQ))
      CALL NEXTF('upper-average-distance=',UPLA(NGRUPQ))
      ELSE IF (WD(1:4).EQ.'WPLA') THEN
      CALL NEXTF('Weight distance cen2cen=',WPLA(NGRUPQ))
C cccccccccccccccccccccccccccccccccccccccccccccccccccccc
C
      ELSE
      CALL CHKEND('STAK-GROUp>',DONE)
      END IF
      END IF
      END DO
      DONE=.FALSE.
C end group parsing: check whether we've done anything sensible
      IF (NATSTK(NGRUPQ).LT.4) THEN
      CALL WRNDIE(-1,'STAK',
     & 'Fewer than 4 atoms selected in sela!')
      NGRUPQ=NGRUPQ-1
      ELSE IF (NATST2(NGRUPQ).LT.4) THEN
      CALL WRNDIE(-1,'STAK',
     & 'Fewer than 4 atoms selected in selb!')
      NGRUPQ=NGRUPQ-1
      ELSE
C allocate permanent space on HEAP for list of atom pointers
      LPSSTP(NGRUPQ)=NATSTK(NGRUPQ)
      HPSSTP(NGRUPQ)=ALLHP(INTEG4(LPSSTP(NGRUPQ)))
C
      LPSST2(NGRUPQ)=NATST2(NGRUPQ)
      HPSST2(NGRUPQ)=ALLHP(INTEG4(LPSST2(NGRUPQ)))
C
C then copy temporary heap pointer into permanent list
      CALL COPYI4(POINTR,HEAP(HPSSTP(NGRUPQ)),
     &            NATSTK(NGRUPQ))
C
      CALL COPYI4(POINT2,HEAP(HPSST2(NGRUPQ)),
     &            NATST2(NGRUPQ))
C
      END IF
C
C======================================================================
      ELSE IF (WD(1:4).EQ.'?   ') THEN
      IF (NGRUPQ.EQ.0) THEN
      WRITE(6,'(A)') ' No STAK groups are defined!!!'
      ELSE
C
C      DO I=1,NGRUPQ
      WRITE(6,'(/,A,I4,A,/,A,I6,/,A,I6,/,A,F10.2,A/)')
     &' STAK GROUP NO. ',NGRUPQ,':',
     &' Number of atoms in sela= ', NATSTK(NGRUPQ),
     &' Number of atoms in selb= ', NATST2(NGRUPQ),
     &' Effective force constant for STAK positional restraints =',
     &  WTSTK(NGRUPQ),' Kcal/mol-A**2'
C      END DO
C
      END IF
C=====================================================================
      ELSE
      CALL CHKEND('STAK>',DONE)
      END IF
      END IF
      END DO
      DONE=.FALSE.
C
      RETURN
      END
C----------------------------------------------------------------
      SUBROUTINE STKFIN
C
C Routine allocates space for restraints group lists on heap.
C
C Author: Martin Laurberg. Based on PLNFIN: Axel T. Brunger
C
      IMPLICIT NONE
C I/O
      INCLUDE 'stack.inc'
      INCLUDE 'funct.inc'
C local
      INTEGER K
C begin
      DO K=1,MAXSGR
      IF (HPSSTP(K).NE.0) THEN
      CALL FREHP(HPSSTP(K),INTEG4(LPSSTP(K)))
      LPSSTP(K)=0
      HPSSTP(K)=0
      END IF
C
      IF (HPSST2(K).NE.0) THEN
      CALL FREHP(HPSST2(K),INTEG4(LPSST2(K)))
      LPSST2(K)=0
      HPSST2(K)=0
      END IF
C
      END DO
      RETURN
      END
C=====================================================================
      SUBROUTINE ENESTK(ESTK)
C
C Compute planarity restraint energy
C Loop over all groups to allocate space on heap for the
C temporary arrays needed.
C Author: Martin Laurberg - Based on ENEPLN: Peter Schultze
C
      IMPLICIT NONE
C I/O
      INCLUDE 'cns.inc'
      INCLUDE 'heap.inc'
      INCLUDE 'funct.inc'
      INCLUDE 'stack.inc'
      DOUBLE PRECISION ESTK
C local
      INTEGER K
      DOUBLE PRECISION DBPREC
      DOUBLE COMPLEX DCVAL
C parameter
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO=0.0D0)
C
C begin
      ESTK=ZERO
C
C loop over all groups
      DO K=1,NGRUPQ
C
      CALL ENEST2(ESTK,HEAP(HPSSTP(K)),NATSTK(K),
     &                 HEAP(HPSST2(K)),NATST2(K),WTSTK(K),
     &                 DCEN(K),LCEN(K),UCEN(K),WCEN(K),
     &                 DPLA(K),LPLA(K),UPLA(K),WPLA(K) )
C
      END DO
C make symbols for number of restraints (groups) NPLAN and
      DBPREC = NGRUPQ
      CALL DECLAR( 'NUMBER', 'DP', '  ', DCVAL, DBPREC)
C
      RETURN
      END
C=====================================================================
      SUBROUTINE TENSOR(POINTR,NATSTL,CENTER,EIGVEC,EIGVAL)
C
C Calculate geometric center,geometric moment of inertia tensor and eigen values.
C Author: Martin Laurberg - Nothing new, based on other parts of the CNS source.
C
      IMPLICIT NONE
C I/O
      INCLUDE 'cns.inc'
      INCLUDE 'consta.inc'
      INCLUDE 'coord.inc'
      INCLUDE 'deriv.inc'
      INTEGER POINTR(*),NATSTL
C local
      DOUBLE PRECISION XAVR, YAVR, ZAVR, XDIF, YDIF, ZDIF
      DOUBLE PRECISION CONST, DIST
      DOUBLE PRECISION T(3,3),CENTER(3), EIGVAL(3), EIGVEC(3,3)
      INTEGER M,MINEIG
      DOUBLE COMPLEX DCVAL
C parameter
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO=0.0D0)
C
C Calculate geometric centre of mass
      XAVR=ZERO
      YAVR=ZERO
      ZAVR=ZERO
C
      DO M=1,NATSTL
      XAVR=XAVR+X(POINTR(M))
      YAVR=YAVR+Y(POINTR(M))
      ZAVR=ZAVR+Z(POINTR(M))
      END DO
C
      XAVR=XAVR/NATSTL
      YAVR=YAVR/NATSTL
      ZAVR=ZAVR/NATSTL
C
      CENTER(1)=XAVR
      CENTER(2)=YAVR
      CENTER(3)=ZAVR
C set up the moment of inertia tensor
      T(1,1)=ZERO
      T(1,2)=ZERO
      T(1,3)=ZERO
      T(2,1)=ZERO
      T(2,2)=ZERO
      T(2,3)=ZERO
      T(3,1)=ZERO
      T(3,2)=ZERO
      T(3,3)=ZERO
C
      DO M=1,NATSTL
      XDIF=X(POINTR(M))-XAVR
      YDIF=Y(POINTR(M))-YAVR
      ZDIF=Z(POINTR(M))-ZAVR
C
      T(1,1)=T(1,1)+X(POINTR(M))*XDIF
      T(1,2)=T(1,2)+X(POINTR(M))*YDIF
      T(1,3)=T(1,3)+X(POINTR(M))*ZDIF
      T(2,1)=T(2,1)+Y(POINTR(M))*XDIF
      T(2,2)=T(2,2)+Y(POINTR(M))*YDIF
      T(2,3)=T(2,3)+Y(POINTR(M))*ZDIF
      T(3,1)=T(3,1)+Z(POINTR(M))*XDIF
      T(3,2)=T(3,2)+Z(POINTR(M))*YDIF
      T(3,3)=T(3,3)+Z(POINTR(M))*ZDIF
      END DO
C
      CALL DIAGSQ(3,3,T,EIGVEC,EIGVAL)
C
      RETURN
      END
C=====================================================================
      SUBROUTINE ECEN (ET,POINTR,NCEN,CENTER,CENTE2,
     &                       DCEN,LCEN,UCEN,WCEN)
C
C Calculate energy and derivative for center to center distance
C   ET     total energy
C   Pointer array - runs from 1 to NCEN
C   NCEN number of atoms in pointer array.
C   CENTER center for selection A
C Author: Martin Laurberg - gosh I hope I'm done soon..
C
      IMPLICIT NONE
C I/O
      INCLUDE 'cns.inc'
      INCLUDE 'timer.inc'
      INCLUDE 'consta.inc'
      INCLUDE 'coord.inc'
      INCLUDE 'deriv.inc'
      INCLUDE 'pick.inc'
      DOUBLE PRECISION ET,DCEN,LCEN,UCEN,WCEN
      INTEGER POINTR(*),  NCEN
      DOUBLE PRECISION CENTER(3),CENTE2(3)
C local
C
      DOUBLE PRECISION XCC,YCC,ZCC,DCC,DEVI,DF
      INTEGER M
C parameter
      DOUBLE PRECISION ZERO,TWO
      PARAMETER (ZERO=0.0D0,TWO=2.0D0)
C begin
C
      ET=ZERO
C
C do square well potential
C ++++++++++++++++++++++++
C
C                   exp           R-(d+dplus)     d+dplus  < R
C E= const ( delta )    , delta = 0               d-dminus < R <  d+dplus
C                                 (d-dminus)-R    R < d-dminus
C
      XCC=CENTER(1)-CENTE2(1)
      YCC=CENTER(2)-CENTE2(2)
      ZCC=CENTER(3)-CENTE2(3)
      DCC=MAX(RSMALL,SQRT(XCC**2 +YCC**2 + ZCC**2))
C
      DEVI=MAX(ZERO,DCC - (DCEN + MAX(ZERO,UCEN))) +
     &     MIN(ZERO,DCC - (DCEN - MAX(ZERO,LCEN)))
C
      ET=WCEN*DEVI**2
      DF=WCEN*TWO*DEVI
C energy calculated from center alone, so derivative per atom
C should be divided by number of atom - perhaps superfluous..
      DF=DF/NCEN
C normalize on length of vector before adding
      DF=DF/DCC

      DO M=1,NCEN
C accumulate derivatives
        DX(POINTR(M))= DX(POINTR(M))+ DF*XCC
        DY(POINTR(M))= DY(POINTR(M))+ DF*YCC
        DZ(POINTR(M))= DZ(POINTR(M))+ DF*ZCC
      END DO
      RETURN
      END
C
C=====================================================================
      SUBROUTINE EPLA (ET,POINTR,NPLA,CENTER,CENTE2,VEC,
     &                       DPLA,LPLA,UPLA,WPLA)
C
C Calculate energy and derivative for center to center distance
C   ET     total energy
C   Pointer array - runs from 1 to NPLA
C   NCEN number of atoms in pointer array.
C   CENTER center for selection A
C Author: Martin Laurberg - more code ahead...
C
      IMPLICIT NONE
C I/O
      INCLUDE 'cns.inc'
      INCLUDE 'timer.inc'
      INCLUDE 'consta.inc'
      INCLUDE 'coord.inc'
      INCLUDE 'deriv.inc'
      INCLUDE 'pick.inc'
      DOUBLE PRECISION ET,DPLA,LPLA,UPLA,WPLA
      INTEGER POINTR(*),  NPLA
      DOUBLE PRECISION CENTER(3),CENTE2(3),VEC(3)
C local
C
      DOUBLE PRECISION DIST,CONST,DEVI,DF
      INTEGER M
C parameter
      DOUBLE PRECISION ZERO,TWO
      PARAMETER (ZERO=0.0D0,TWO=2.0D0)
C begin
C
      ET=ZERO
C
C do square well potential
C ++++++++++++++++++++++++
C
C                   exp           R-(d+dplus)     d+dplus  < R
C E= const ( delta )    , delta = 0               d-dminus < R <  d+dplus
C                                 (d-dminus)-R    R < d-dminus
C
C
C find the constant for the equation of the plane by plugging in
C the centre of mass:
      CONST=VEC(1)*CENTE2(1) +
     &      VEC(2)*CENTE2(2) +
     &      VEC(3)*CENTE2(3)

C distance between center of selection and plane of reference
      DIST=VEC(1)*CENTER(1) +
     &     VEC(2)*CENTER(2) +
     &     VEC(3)*CENTER(3)
     &     - CONST
C
      DEVI=MAX(ZERO,DIST - (DPLA + MAX(ZERO,UPLA))) +
     &     MIN(ZERO,DIST - (DPLA - MAX(ZERO,LPLA)))
C
      ET=WPLA*DEVI**2
      DF=WPLA*TWO*DEVI
C energy calculated from center alone, so derivative per atom
C should be divided by number of atoms
      DF=DF/NPLA
C
      DO M=1,NPLA
C accumulate derivatives
C      WRITE(6,'(A,I4,A,F10.2,A,F10.2)')
C     &' M ',M,' DX ', DX(POINTR(M)),' VEC ',VEC(1)

        DX(POINTR(M))= DX(POINTR(M))+ DF*VEC(1)
        DY(POINTR(M))= DY(POINTR(M))+ DF*VEC(2)
        DZ(POINTR(M))= DZ(POINTR(M))+ DF*VEC(3)
        
C      WRITE(6,'(A,I4,A,F10.2,A,F10.2,/)')
C     &' M ',M,' DX ', DX(POINTR(M)),' DF ',DF
      END DO
      RETURN
      END
C
C=====================================================================
      SUBROUTINE EANVEC(ET,ETV,POINTR,NTHETA,ACTC,ACTB,
     &                  WGHT,VWGHT,CENTER,VEC)
C Calculate energy and derivative for 
C   ET     total energy
C   ETV    ? goes with VWGHT
C   Pointer array - runs from 1 to NTHETA
C   NTHETA number of atoms in pointer array.
C   ACTC   angle energy constant - This version uses one value for all atms
C   ACTB   target angle value in radians. Use PI/2 for RIC RVC perpendicular
C   WGHT   ? Hamiltonian weight something.. - see PIGWGHT in ener.inc
C   VWGHT  ? perturbing potential           - see PIGAVWT
C   CENTER center for rotation
C   VEC    reference vector. Rotation is in the plane spanned by RIC and RVC 
C Author: Martin Laurberg - Gleaning code around, mainly eangle.f
C
      IMPLICIT NONE
C I/O
      INCLUDE 'cns.inc'
      INCLUDE 'timer.inc'
      INCLUDE 'consta.inc'
      INCLUDE 'coord.inc'
      INCLUDE 'deriv.inc'
      INCLUDE 'pick.inc'
      DOUBLE PRECISION ET,ETV
      INTEGER POINTR(*),  NTHETA
      DOUBLE PRECISION WGHT,VWGHT
      DOUBLE PRECISION ACTC, ACTB,CENTER(3),VEC(3)
C local
      DOUBLE PRECISION PHI, SP, DI, DF, E, RECSP, CP
      DOUBLE PRECISION RICX,RICY,RICZ,RIC,RVCX,RVCY,RVCZ,RVC
CCC      DOUBLE PRECISION WARNG
      DOUBLE PRECISION WTWO
C
      DOUBLE PRECISION DPRICX, DPRICY, DPRICZ
      DOUBLE PRECISION DPRVCX, DPRVCY, DPRVCZ
      INTEGER M
C
C parameter
      DOUBLE PRECISION ZERO, ONE, TWO, MCONST, RADI
      PARAMETER (ZERO=0.0D0, ONE=1.0D0, TWO=2.0D0, RADI=180.0D0/PI)
C===>parameter MCONST truncates the reciprocals and makes sure
C===>we don't devide by zero
      PARAMETER (MCONST=0.0001D0)

C initialization
      ET=0.0D0
      ETV=0.0D0
CCC      WARNG=0.0D0
      WTWO=TWO*WGHT
C

      DO M=1,NTHETA
C first compute the difference RIC=RI-RC
      RICX = X(POINTR(M))-CENTER(1)
      RICY = Y(POINTR(M))-CENTER(2)
      RICZ = Z(POINTR(M))-CENTER(3)
C compute the norm of RIC, VEC and set to MCONST if it is too small.
      RIC=ONE/SQRT(MAX(MCONST,RICX**2+RICY**2+RICZ**2))
      RVC=ONE/SQRT(MAX(MCONST,VEC(1)**2+VEC(2)**2+VEC(3)**2))
C
C normalize RIC, RVC - I guess VEC is usually already normalized, but whatever..
      RICX = RICX*RIC
      RICY = RICY*RIC
      RICZ = RICZ*RIC
      RVCX = VEC(1)*RVC
      RVCY = VEC(2)*RVC
      RVCZ = VEC(3)*RVC
C compute CP = cos( phi )
      CP=RICX*RVCX+RICY*RVCY+RICZ*RVCZ
C
C compute SP = sin( phi )   ( make sure  CP within boundaries )
      SP=SQRT(MAX(ZERO,ONE-CP**2))
C
C compute PHI ( make sure  CP within boundaries )
      PHI=ACOS(MIN(ONE,MAX(-ONE,CP)))
C
C======================================================================
C==> compute function (phi) and derivative
C==> replace this part if you want to use another functional form
C
C harmonic function: const* ( phi - phi0 )**2
      DI=PHI-ACTB
      DF=ACTC*DI
      E=DF*DI
      DF=WTWO*DF
C======================================================================
C
C accumulate energy
      ET=ET+E
C
C compute reciprocal of SP multiplied by the derivative of the
C function DF
      RECSP=DF/MAX(MCONST,SP)
C
C compute:
C    d (phi)
C DF -------,  etc.
C    d rij
C
      DPRICX=RECSP*RIC*(CP*RICX-RVCX)
      DPRICY=RECSP*RIC*(CP*RICY-RVCY)
      DPRICZ=RECSP*RIC*(CP*RICZ-RVCZ)
      DPRVCX=RECSP*RVC*(CP*RVCX-RICX)
      DPRVCY=RECSP*RVC*(CP*RVCY-RICY)
      DPRVCZ=RECSP*RVC*(CP*RVCZ-RICZ)
C
C now scatter forces 
C
      DX(POINTR(M))=DX(POINTR(M))+DPRICX
      DY(POINTR(M))=DY(POINTR(M))+DPRICY
      DZ(POINTR(M))=DZ(POINTR(M))+DPRICZ
      END DO
C
      ETV=ET*VWGHT
      ET=ET*WGHT
C
      RETURN
      END
C
C=====================================================================
      SUBROUTINE ENEST2(ESTK,POINTR,NATSTK,
     &                       POINT2,NATST2,WTSTK,
     &                       DCEN,LCEN,UCEN,WCEN,
     &                       DPLA,LPLA,UPLA,WPLA)
C
C Effective energy and derivatives for planarity restraints
C Author: Martin Laurberg. Partly from ENEPL2: Peter Schultze
C
      IMPLICIT NONE
C I/O
      INCLUDE 'cns.inc'
      INCLUDE 'consta.inc'
      INCLUDE 'coord.inc'
      INCLUDE 'deriv.inc'
      DOUBLE PRECISION ESTK
      INTEGER POINTR(*),  NATSTK,POINT2(*),  NATST2
      DOUBLE PRECISION WTSTK,DCEN,LCEN,UCEN,WCEN
      DOUBLE PRECISION DPLA,LPLA,UPLA,WPLA
C local
      DOUBLE PRECISION XAVR, YAVR, ZAVR, XDIF, YDIF, ZDIF,
     &                 XAVR2,YAVR2,ZAVR2,XDIF2,YDIF2,ZDIF2
      DOUBLE PRECISION CONST, DIST,CONST2, DIST2
      DOUBLE PRECISION T(3,3), EIGVAL(3), EIGVEC(3,3),CENTER(3),
     &                 T2(3,3), EIGVA2(3), EIGVE2(3,3),CENTE2(3)
      DOUBLE PRECISION STKVEC(3),STKVE2(3),ET,ETV,ET2,ETV2
      DOUBLE PRECISION ECT,ECT2,EPT,EPT2
      INTEGER M,MINEIG,MINEI2
      DOUBLE COMPLEX DCVAL
C parameter
      DOUBLE PRECISION ZERO,ONE,TEN,NINETY,RADI,ACTC,ACTB
      PARAMETER (ZERO=0.0D0, ONE=1.0D0, TEN=10.0D0)
      PARAMETER (NINETY=90.0D0,RADI=180.0D0/PI)
      PARAMETER (ACTC=2.0D0,ACTB=PI/2.0D0)
C
C
C Calculate geometric center, tensor and eigen values for selection A
      CALL TENSOR(POINTR,NATSTK,CENTER,EIGVEC,EIGVAL) 
C the lowest Eigenvalue of T equals the sum of the squares of the orthogonal
C distances between atoms and plane. The corresponding Eigenvector
C is the normvector of the plane. Do the diagonalisation:
C
C find pointer to the lowest Eigenvalue
      MINEIG=1
      IF( EIGVAL(2) .LT. EIGVAL(MINEIG) ) THEN
      MINEIG=2
      END IF
      IF( EIGVAL(3) .LT. EIGVAL(MINEIG) ) THEN
      MINEIG=3
      END IF
C
C This is the normal vector to the plane spannned by selection A
      STKVEC(1)=EIGVEC(1,MINEIG)
      STKVEC(2)=EIGVEC(2,MINEIG)
      STKVEC(3)=EIGVEC(3,MINEIG)

C Calculate geometric center, tensor and eigen values for selection B
      CALL TENSOR(POINT2,NATST2,CENTE2,EIGVE2,EIGVA2) 
      MINEI2=1
      IF( EIGVA2(2) .LT. EIGVA2(MINEI2) ) THEN
      MINEI2=2
      END IF
      IF( EIGVA2(3) .LT. EIGVA2(MINEI2) ) THEN
      MINEI2=3
      END IF
C This is the normal vector to the plane spannned by selection B
      STKVE2(1)=EIGVE2(1,MINEI2)
      STKVE2(2)=EIGVE2(2,MINEI2)
      STKVE2(3)=EIGVE2(3,MINEI2)


C Calculate angular energy for selection A using normal vector to selection B
      CALL EANVEC(ET,ETV,POINTR,NATSTK,ACTC,ACTB,
     &                  WTSTK,ONE,CENTER,STKVE2)

C Calculate angular energy for selection B using normal vector to selection A
      CALL EANVEC(ET2,ETV2,POINT2,NATST2,ACTC,ACTB,
     &                  ONE,ONE,CENTE2,STKVEC)
C
C
C Calculate Distance energy for center of selection A to center of selection B
      CALL ECEN (ECT,POINTR,NATSTK,CENTER,CENTE2,DCEN,LCEN,UCEN,WCEN)  
C Calculate Distance energy for center of selection B to center of selection A
      CALL ECEN (ECT2,POINT2,NATST2,CENTE2,CENTER,DCEN,LCEN,UCEN,WCEN) 
C
C Calculate Distance energy for center of selection A to plane of selection B
      CALL EPLA (EPT,POINTR,NATSTK,CENTER,CENTE2,STKVE2,
     &           DPLA,LPLA,UPLA,WPLA)
      CALL EPLA (EPT2,POINT2,NATST2,CENTE2,CENTER,STKVEC,
     &           DPLA,LPLA,UPLA,WPLA)

C cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C determine total energy
      ESTK=ESTK + ET + ET2 +
     &            ECT + ECT2 +
     &            EPT + EPT2 
C
C make symbols of the components of the normvector:
      CALL DECLAR( 'STKAX', 'DP', '  ', DCVAL, EIGVEC(1,MINEIG) )
      CALL DECLAR( 'STKAY', 'DP', '  ', DCVAL, EIGVEC(2,MINEIG) )
      CALL DECLAR( 'STKAZ', 'DP', '  ', DCVAL, EIGVEC(3,MINEIG) )
C
      CALL DECLAR( 'STKBX', 'DP', '  ', DCVAL, EIGVE2(1,MINEI2) )
      CALL DECLAR( 'STKBY', 'DP', '  ', DCVAL, EIGVE2(2,MINEI2) )
      CALL DECLAR( 'STKBZ', 'DP', '  ', DCVAL, EIGVE2(3,MINEI2) )
C
      CALL DECLAR( 'STCAX', 'DP', '  ', DCVAL, CENTER(1) )
      CALL DECLAR( 'STCAY', 'DP', '  ', DCVAL, CENTER(2) )
      CALL DECLAR( 'STCAZ', 'DP', '  ', DCVAL, CENTER(3) )
      CALL DECLAR( 'STCBX', 'DP', '  ', DCVAL, CENTE2(1) )
      CALL DECLAR( 'STCBY', 'DP', '  ', DCVAL, CENTE2(2) )
      CALL DECLAR( 'STCBZ', 'DP', '  ', DCVAL, CENTE2(3) )
      CALL DECLAR( 'ECT', 'DP', '  ', DCVAL, ECT )
      CALL DECLAR( 'ECT2', 'DP', '  ', DCVAL, ECT2 )
      CALL DECLAR( 'DCEN', 'DP', '  ', DCVAL, DCEN )
      CALL DECLAR( 'LCEN', 'DP', '  ', DCVAL, LCEN )
      CALL DECLAR( 'UCEN', 'DP', '  ', DCVAL, UCEN )
      CALL DECLAR( 'WCEN', 'DP', '  ', DCVAL, WCEN )
      RETURN
      END

C=====================================================================
